@@ -11,7 +11,7 @@ GOLDEN = DATA / "golden"
 
 
 @pytest.mark.parametrize(
-    ["args", "golden_file"],
+    ("args", "golden_file"),
     [
         (["-p", "kajiki_test_data.kitchensink"], "kitchensink1.html"),
         ([str(DATA / "kitchensink.html")], "kitchensink1.html"),
@@ -29,7 +29,7 @@ def test_golden_file(args, golden_file, capsys):
 
 
 def test_file_not_found():
-    with pytest.raises(IOError):
+    with pytest.raises(FileNotFoundError):
         main(["/does/not/exist.txt"])
 
 
