@@ -1,11 +1,11 @@
-#!/usr/bin/env python
-
 import os
 import sys
 import traceback
 import xml.dom.minidom
 from io import BytesIO
-from unittest import TestCase, main
+from unittest import TestCase
+
+import pytest
 
 import kajiki
 from kajiki import FileLoader, MockLoader, PackageLoader, XMLTemplate, i18n
@@ -64,28 +64,27 @@ class TestExpand(TestCase):
             if node.tagName == "div":
                 node = node.childNodes[0]
                 continue
-            assert node.tagName == tagname, "%s != %s" % (node.tagName, tagname)
+            assert node.tagName == tagname, f"{node.tagName} != {tagname}"
             if attr:
-                assert len(node.attributes) == 1
-                assert node.hasAttribute(attr)
-                assert node.getAttribute(attr) == tagname.split(":")[-1]
+                if node.tagName != "py:case":
+                    assert len(node.attributes) == 1, node.attributes.items()
+                    assert node.hasAttribute(attr)
+                    assert node.getAttribute(attr) == tagname.split(":")[-1]
+                else:
+                    assert len(node.attributes) == 2
             else:
                 assert len(node.attributes) == 0
             assert len(node.childNodes) == 1
             node = node.childNodes[0]
 
 
-def perform(source, expected_output, context=dict(name="Rick"), **options):
+def perform(source, expected_output, context=None, **options):
+    context = context or {"name": "Rick"}
     tpl = XMLTemplate(source, **options)
-    try:
-        rsp = tpl(context).render()
-        assert isinstance(rsp, str), "render() must return a unicode string."
-        assert rsp == expected_output, (rsp, expected_output)
-    except Exception as e:
-        print("\n" + tpl.py_text)
-        raise e
-    else:
-        return tpl
+    rsp = tpl(context).render()
+    assert isinstance(rsp, str), "render() must return a string."
+    assert rsp == expected_output, (rsp, expected_output)
+    return tpl
 
 
 class TestSimple(TestCase):
@@ -102,9 +101,7 @@ class TestSimple(TestCase):
         perform(src, src, mode="xml")
 
     def test_textarea_whitespace(self):
-        src = (
-            '<textarea name="foo">\nHey there.  \n\n    I am indented.\n' "</textarea>"
-        )
+        src = '<textarea name="foo">\nHey there.  \n\n    I am indented.\n' "</textarea>"
         perform(src, src, mode="html")
         perform(src, src, mode="xml")
 
@@ -120,21 +117,15 @@ class TestSimple(TestCase):
         must be explicitly be made so.
         """
         script = 'if (1 < 2) { doc.write("<p>Offen&nbsp;bach</p>"); }\n'
-        src = "<script><![CDATA[\n{0}]]></script>".format(script)
-        perform(
-            src, mode="html", expected_output="<script>\n{0}</script>".format(script)
-        )
-        perform(
-            src, "<script>/*<![CDATA[*/\n{0}/*]]>*/</script>".format(script), mode="xml"
-        )
+        src = f"<script><![CDATA[\n{script}]]></script>"
+        perform(src, mode="html", expected_output=f"<script>\n{script}</script>")
+        perform(src, f"<script>/*<![CDATA[*/\n{script}/*]]>*/</script>", mode="xml")
 
     def test_style_escaping(self):
         style = "html > body { display: none; }\n"
-        src = "<style><![CDATA[\n{0}]]></style>".format(style)
-        perform(
-            src, "<style>/*<![CDATA[*/\n{0}/*]]>*/</style>".format(style), mode="xml"
-        )
-        perform(src, "<style>\n{0}</style>".format(style), mode="html")
+        src = f"<style><![CDATA[\n{style}]]></style>"
+        perform(src, f"<style>/*<![CDATA[*/\n{style}/*]]>*/</style>", mode="xml")
+        perform(src, f"<style>\n{style}</style>", mode="html")
 
     def test_script_variable(self):
         """Interpolate variables inside <script> tags"""
@@ -142,21 +133,17 @@ class TestSimple(TestCase):
         perform(src, "<script>/*<![CDATA[*/ Rick /*]]>*/</script>", mode="xml")
         perform(src, "<script> Rick </script>", mode="html")
 
-    def test_CDATA_disabled(self):
+    def test_cdata_disabled(self):
         src = "<script> $name </script>"
         perform(src, "<script> Rick </script>", mode="xml", cdata_scripts=False)
         perform(src, "<script> Rick </script>", mode="html", cdata_scripts=False)
 
-    def test_CDATA_escaping(self):
+    def test_cdata_escaping(self):
         src = """<myxml><data><![CDATA[&gt;&#240; $name]]></data></myxml>"""
-        perform(
-            src, "<myxml><data><![CDATA[&gt;&#240; Rick]]></data></myxml>", mode="xml"
-        )
-        perform(
-            src, "<myxml><data><![CDATA[&gt;&#240; Rick]]></data></myxml>", mode="html"
-        )
+        perform(src, "<myxml><data><![CDATA[&gt;&#240; Rick]]></data></myxml>", mode="xml")
+        perform(src, "<myxml><data><![CDATA[&gt;&#240; Rick]]></data></myxml>", mode="html")
 
-    def test_CDATA_escaping_mixed(self):
+    def test_cdata_escaping_mixed(self):
         src = """<myxml><data><![CDATA[&gt;&#240; $name]]> &gt;</data></myxml>"""
         perform(
             src,
@@ -169,17 +156,17 @@ class TestSimple(TestCase):
             mode="html",
         )
 
-    def test_script_commented_CDATA(self):
+    def test_script_commented_cdata(self):
         script = 'if (1 < 2) { doc.write("<p>Offen&nbsp;bach</p>"); }\n'
-        src = "<script>/*<![CDATA[*/\n{0}/*]]>*/</script>".format(script)
+        src = f"<script>/*<![CDATA[*/\n{script}/*]]>*/</script>"
         perform(
             src,
             mode="html",
-            expected_output="<script>/**/\n{0}/**/</script>".format(script),
+            expected_output=f"<script>/**/\n{script}/**/</script>",
         )
         perform(
             src,
-            "<script>/*<![CDATA[*//**/\n{0}/**//*]]>*/</script>".format(script),
+            f"<script>/*<![CDATA[*//**/\n{script}/**//*]]>*/</script>",
             mode="xml",
         )
 
@@ -212,16 +199,15 @@ class TestSimple(TestCase):
             "<div>Hello, Rick</div>",
         )
 
-    def test_expr_multiline_and_IndentationError(self):
-        try:
+    def test_expr_multiline_and_indentation_error(self):
+        with pytest.raises(XMLTemplateCompileError) as e:
             XMLTemplate(
                 """<div>Hello, ${ 'pippo' +
                 'baudo'}</div>"""
             )().render()
-        except XMLTemplateCompileError as e:
-            assert "`'pippo' +\n                'baudo'`" in str(e), str(e)
-            assert "Hello" in str(e)
-            assert "baudo" in str(e)
+        assert "`'pippo' +\n                'baudo'`" in str(e.value)
+        assert "Hello" in str(e.value)
+        assert "baudo" in str(e.value)
 
     def test_expr_multiline_cdata(self):
         perform(
@@ -238,13 +224,7 @@ class TestSimple(TestCase):
         """
         js = "$(function () { alert('.ready()'); });"
         src = "<html><pre>" + js + "</pre><script>" + js + "</script></html>"
-        out = (
-            "<html><pre>"
-            + js
-            + "</pre><script>/*<![CDATA[*/"
-            + js
-            + "/*]]>*/</script></html>"
-        )
+        out = "<html><pre>" + js + "</pre><script>/*<![CDATA[*/" + js + "/*]]>*/</script></html>"
         perform(src, out)
 
     def test_jquery_shortcut_is_not_expr(self):
@@ -252,13 +232,7 @@ class TestSimple(TestCase):
 
         js = "$.extend({}, {foo: 'bar'})"
         src = "<html><pre>" + js + "</pre><script>" + js + "</script></html>"
-        out = (
-            "<html><pre>"
-            + js
-            + "</pre><script>/*<![CDATA[*/"
-            + js
-            + "/*]]>*/</script></html>"
-        )
+        out = "<html><pre>" + js + "</pre><script>/*<![CDATA[*/" + js + "/*]]>*/</script></html>"
         perform(src, out)
 
     def test_xml_entities(self):
@@ -267,7 +241,7 @@ class TestSimple(TestCase):
 
     def test_html_entities(self):
         source = "<div>Spam&nbsp;Spam &lt; Spam &gt; Spam &hellip;</div>"
-        output = "<div>Spam Spam &lt; Spam &gt; Spam \u2026</div>"
+        output = "<div>Spam\xa0Spam &lt; Spam &gt; Spam \u2026</div>"
         assert chr(32) in output  # normal space
         assert chr(160) in output  # non breaking space
         perform(source, output)
@@ -318,7 +292,7 @@ $i is <py:switch test="i % 4">
         )
 
     def test_switch_div(self):
-        try:
+        with pytest.raises(XMLTemplateCompileError) as e:
             perform(
                 """
         <div class="test" py:switch="5 == 3">
@@ -327,18 +301,61 @@ $i is <py:switch test="i % 4">
         </div>""",
                 "<div><div>False</div></div>",
             )
-        except XMLTemplateCompileError as e:
-            self.assertTrue(
-                "py:switch directive can only contain py:case and py:else nodes"
-                in str(e)
+        assert "py:switch directive can only contain py:case and py:else nodes" in str(e)
+
+
+class TestMatch:
+    def setup_class(self):
+        if sys.version_info < (3, 10):
+            pytest.skip("pep622 unavailable before python3.10")
+
+    def test_match(self):
+        perform(
+            """<div py:for="i in range(2)">
+$i is <py:match on="i % 2">
+<py:case match="0">even</py:case>
+<py:case match="_">odd</py:case>
+</py:match></div>""",
+            """<div>
+0 is even</div><div>
+1 is odd</div>""",
+        )
+
+    def test_match_div(self):
+        with pytest.raises(
+            XMLTemplateCompileError,
+            match="case must have either value or match attribute, the former for py:switch, the latter for py:match",
+        ):
+            perform(
+                """
+        <div class="test" py:match="5 == 3">
+            <p py:case="True">True</p>
+            <p py:case="_">False</p>
+        </div>""",
+                "<div><div>False</div></div>",
+            )
+
+    def test_match_aliens(self):
+        with pytest.raises(
+            XMLTemplateCompileError,
+            match="py:match directive can only contain py:case",
+        ):
+            perform(
+                """<div py:for="i in range(2)">
+$i is <py:match on="i % 2">
+alien
+<py:case match="0">even</py:case>
+<py:case match="_">odd</py:case>
+</py:match></div>""",
+                """<div>
+0 is even</div><div>
+1 is odd</div>""",
             )
-        else:
-            self.assertTrue(False, msg="Should have raised XMLTemplateParseError")
 
 
 class TestElse(TestCase):
     def test_pyif_pyelse(self):
-        try:
+        with pytest.raises(XMLTemplateCompileError) as e:
             perform(
                 """
             <div>
@@ -347,13 +364,7 @@ class TestElse(TestCase):
             </div>""",
                 """<div>False</div>""",
             )
-        except XMLTemplateCompileError as e:
-            self.assertTrue(
-                "py:else directive must be inside a py:switch or directly after py:if"
-                in str(e)
-            )
-        else:
-            self.assertTrue(False, msg="Should have raised XMLTemplateParseError")
+        assert "py:else directive must be inside a py:switch or directly after py:if" in str(e)
 
     def test_pyiftag_pyelse_continuation(self):
         perform(
@@ -409,8 +420,7 @@ class TestWith(TestCase):
 
     def test_with_ordered_multiple(self):
         perform(
-            """<div py:with="a='foo';b=a * 2;c=b[::-1];d=c[:3]">"""
-            """$a $b $c $d</div>""",
+            """<div py:with="a='foo';b=a * 2;c=b[::-1];d=c[:3]">""" """$a $b $c $d</div>""",
             "<div>foo foofoo oofoof oof</div>",
         )
 
@@ -498,7 +508,7 @@ class TestImport(TestCase):
             }
         )
         tpl = loader.import_("tpl.html")
-        rsp = tpl(dict(name="Rick")).render()
+        rsp = tpl({"name": "Rick"}).render()
         assert (
             rsp
             == """<div>
@@ -544,7 +554,7 @@ class TestImport(TestCase):
             }
         )
         tpl = loader.import_("tpl.html")
-        rsp = tpl(dict(name="Rick")).render()
+        rsp = tpl({"name": "Rick"}).render()
         assert (
             rsp
             == """<div>
@@ -572,17 +582,16 @@ class TestImport(TestCase):
                     '${value_of("name")}</p>\n'
                 ),
                 "tpl.html": XMLTemplate(
-                    "<html><body><p>This is the body</p>\n"
-                    '<py:include href="included.html"/></body></html>'
+                    "<html><body><p>This is the body</p>\n" '<py:include href="included.html"/></body></html>'
                 ),
             }
         )
         tpl = loader.import_("tpl.html")
-        rsp = tpl(dict(name="Rick")).render()
+        rsp = tpl({"name": "Rick"}).render()
         assert (
-            "<html><body><p>This is the body</p>\n"
+            rsp == "<html><body><p>This is the body</p>\n"
             "<p>The included template must also access Kajiki globals and "
-            "the template context: Rick</p></body></html>" == rsp
+            "the template context: Rick</p></body></html>"
         )
 
     def test_include_html5(self):
@@ -600,26 +609,24 @@ class TestImport(TestCase):
                 self.sources = sources
                 super().__init__({})
 
-            def _load(self, name, encoding="utf-8", *args, **kwargs):
-                return XMLTemplate(
-                    source=self.sources[name], mode="html5", *args, **kwargs
-                )
+            def _load(self, name, encoding="utf-8", **kwargs):
+                del encoding
+                return XMLTemplate(source=self.sources[name], mode="html5", **kwargs)
 
         loader = XMLSourceLoader(
             {
                 "included.html": "<p>The included template must also "
                 "access Kajiki globals and the template context: "
                 '${value_of("name")}</p>\n',
-                "tpl.html": "<html><body><p>This is the body</p>\n"
-                '<py:include href="included.html"/></body></html>',
+                "tpl.html": "<html><body><p>This is the body</p>\n" '<py:include href="included.html"/></body></html>',
             }
         )
         tpl = loader.import_("tpl.html")
-        rsp = tpl(dict(name="Rick")).render()
+        rsp = tpl({"name": "Rick"}).render()
         assert (
-            "<!DOCTYPE html>\n<html><body><p>This is the body</p>\n"
+            rsp == "<!DOCTYPE html>\n<html><body><p>This is the body</p>\n"
             "<p>The included template must also access Kajiki globals and "
-            "the template context: Rick</p></body></html>" == rsp
+            "the template context: Rick</p></body></html>"
         ), rsp
 
 
@@ -658,7 +665,7 @@ ${parent.body()}
             }
         )
         tpl = loader.import_("child.html")
-        rsp = tpl(dict(name="Rick")).render()
+        rsp = tpl({"name": "Rick"}).render()
         assert (
             rsp
             == """<div>
@@ -691,9 +698,9 @@ child.id() = <span>mid</span>
             }
         )
         tpl = loader.import_("child.html")
-        rsp = tpl(dict(p=0)).render()
+        rsp = tpl({"p": 0}).render()
         assert rsp == "<div><span>Parent 0</span></div>", rsp
-        rsp = tpl(dict(p=1)).render()
+        rsp = tpl({"p": 1}).render()
         assert rsp == "<div><span>Parent 1</span></div>", rsp
 
     def test_block(self):
@@ -913,8 +920,7 @@ import os
 class TestComment(TestCase):
     def test_basic(self):
         perform(
-            "<div><!-- This comment is preserved. -->"
-            "<!--! This comment is stripped. --></div>",
+            "<div><!-- This comment is preserved. -->" "<!--! This comment is stripped. --></div>",
             "<div><!--  This comment is preserved.  --></div>",
         )
 
@@ -938,29 +944,29 @@ class TestAttributes(TestCase):
         perform('<div py:attrs="dict(checked=None)"/>', "<div/>")
 
     def test_strip(self):
-        TPL = '<div><h1 py:strip="header">Header</h1></div>'
-        perform(TPL, "<div>Header</div>", context=dict(header=True))
-        perform(TPL, "<div><h1>Header</h1></div>", context=dict(header=False))
-        TPL = """<div><p py:strip="">It's...</p></div>"""
-        perform(TPL, "<div>It's...</div>")
+        tpl = '<div><h1 py:strip="header">Header</h1></div>'
+        perform(tpl, "<div>Header</div>", context={"header": True})
+        perform(tpl, "<div><h1>Header</h1></div>", context={"header": False})
+        tpl = """<div><p py:strip="">It's...</p></div>"""
+        perform(tpl, "<div>It's...</div>")
 
     def test_html_attrs(self):
-        TPL = '<input type="checkbox" checked="$checked"/>'
-        context0 = dict(checked=None)
-        context1 = dict(checked=True)
-        perform(TPL, '<input type="checkbox"/>', context0, mode="xml")
-        perform(TPL, '<input checked="True" type="checkbox"/>', context1, mode="xml")
-        perform(TPL, '<input type="checkbox">', context0, mode="html")
-        perform(TPL, '<input checked type="checkbox">', context1, mode="html")
+        tpl = '<input type="checkbox" checked="$checked"/>'
+        context0 = {"checked": None}
+        context1 = {"checked": True}
+        perform(tpl, '<input type="checkbox"/>', context0, mode="xml")
+        perform(tpl, '<input checked="True" type="checkbox"/>', context1, mode="xml")
+        perform(tpl, '<input type="checkbox">', context0, mode="html")
+        perform(tpl, '<input checked type="checkbox">', context1, mode="html")
         perform(
-            TPL,
+            tpl,
             '<!DOCTYPE html>\n<input checked type="checkbox">',
             context1,
             mode="html5",
             is_fragment=False,
         )
         perform(
-            "<!DOCTYPE html>\n" + TPL,
+            "<!DOCTYPE html>\n" + tpl,
             '<!DOCTYPE html>\n<input checked type="checkbox">',
             context1,
             mode=None,
@@ -969,18 +975,15 @@ class TestAttributes(TestCase):
 
     def test_xml_namespaces(self):
         """Namespaced attributes pass through."""
-        TPL = '<p xml:lang="en">English text</p>'
-        perform(TPL, TPL, mode="xml")
-        perform(TPL, TPL, mode="html")
+        tpl = '<p xml:lang="en">English text</p>'
+        perform(tpl, tpl, mode="xml")
+        perform(tpl, tpl, mode="html")
 
     def test_escape_attr_values(self):
         """Escape static and dynamic attribute values."""
-        context = dict(url="https://domain.com/path?a=1&b=2")
+        context = {"url": "https://domain.com/path?a=1&b=2"}
         source = """<a title='"Ha!"' href="$url">Link</a>"""
-        output = (
-            '<a href="https://domain.com/path?a=1&amp;b=2" '
-            'title="&quot;Ha!&quot;">Link</a>'
-        )
+        output = '<a href="https://domain.com/path?a=1&amp;b=2" ' 'title="&quot;Ha!&quot;">Link</a>'
         perform(source, output, context, mode="html")
         perform(source, output, context, mode="xml")
 
@@ -989,18 +992,15 @@ class TestDebug(TestCase):
     def test_debug(self):
         loader = FileLoader(path=os.path.join(os.path.dirname(__file__), "data"))
         tpl = loader.import_("debug.html")
-        try:
+        with pytest.raises(ValueError, match="Test error") as exc_info:
             tpl().render()
-            assert False, "Should have raised ValueError"
-        except ValueError:
-            exc_info = sys.exc_info()
-            stack = traceback.extract_tb(exc_info[2])
+
         # Verify we have stack trace entries in the template
-        for fn, lno, func, line in stack:
-            if fn.endswith("debug.html"):
+        for tb_entry in exc_info.traceback:
+            if tb_entry.path.name == "debug.html":
                 break
         else:
-            assert False, "Stacktrace is all python"
+            pytest.fail("Stacktrace is all python")
 
 
 class TestPackageLoader(TestCase):
@@ -1016,23 +1016,21 @@ class TestBuiltinFunctions(TestCase):
 <div py:if="defined('albatross')">$albatross</div>\
 <p py:if="defined('parrot')">$parrot</p></div>""",
             expected_output="<div><p>Bereft of life, it rests in peace</p></div>",
-            context=dict(parrot="Bereft of life, it rests in peace"),
+            context={"parrot": "Bereft of life, it rests in peace"},
         )
 
     def test_value_of(self):
-        TPL = "<p>${value_of('albatross', 'Albatross!!!')}</p>"
-        perform(TPL, expected_output="<p>It's</p>", context=dict(albatross="It's"))
-        perform(TPL, expected_output="<p>Albatross!!!</p>")
+        tpl = "<p>${value_of('albatross', 'Albatross!!!')}</p>"
+        perform(tpl, expected_output="<p>It's</p>", context={"albatross": "It's"})
+        perform(tpl, expected_output="<p>Albatross!!!</p>")
 
     def test_literal(self):
         """Escape by default; literal() marks as safe."""
-        context = dict(albatross="<em>Albatross!!!</em>")
+        context = {"albatross": "<em>Albatross!!!</em>"}
         expected_output = "<p><em>Albatross!!!</em></p>"
         perform("<p>${literal(albatross)}</p>", expected_output, context)
         perform("<p>${Markup(albatross)}</p>", expected_output, context)
-        perform(
-            "<p>$albatross</p>", "<p>&lt;em&gt;Albatross!!!&lt;/em&gt;</p>", context
-        )
+        perform("<p>$albatross</p>", "<p>&lt;em&gt;Albatross!!!&lt;/em&gt;</p>", context)
         from kajiki.util import literal
 
         markup = '<b>"&amp;"</b>'
@@ -1041,21 +1039,18 @@ class TestBuiltinFunctions(TestCase):
 
 class TestTranslation(TestCase):
     def test_scripts_non_translatable(self):
-        src = (
-            "<xml><div>Hi</div><script>hello world</script>"
-            "<style>hello style</style></xml>"
-        )
+        src = "<xml><div>Hi</div><script>hello world</script>" "<style>hello style</style></xml>"
         doc = _Parser("<string>", src).parse()
 
         for n in _Compiler("<string>", doc).compile():
             text = getattr(n, "text", "")
             if text in ("hello world", "hello style"):
-                self.assertFalse(isinstance(n, TranslatableTextNode))
+                assert not isinstance(n, TranslatableTextNode)
 
         for n in _Compiler("<string>", doc, cdata_scripts=False).compile():
             text = getattr(n, "text", "")
             if text in ("hello world", "hello style"):
-                self.assertFalse(isinstance(n, TranslatableTextNode))
+                assert not isinstance(n, TranslatableTextNode)
 
     def test_extract_translate(self):
         src = """<xml><div>Hi</div><p>
@@ -1072,14 +1067,12 @@ class TestTranslation(TestCase):
         for strip_text in (False, True):
             # Build translation table
             messages = {}
-            for _, _, msgid, _ in i18n.extract(
-                BytesIO(src.encode("utf-8")), None, None, {"strip_text": strip_text}
-            ):
-                messages[msgid] = "TRANSLATED(%s)" % msgid
+            for _, _, msgid, _ in i18n.extract(BytesIO(src.encode("utf-8")), None, None, {"strip_text": strip_text}):
+                messages[msgid] = f"TRANSLATED({msgid})"
 
             # Provide a fake translation function
             default_gettext = i18n.gettext
-            i18n.gettext = lambda s: messages[s]
+            i18n.gettext = messages.__getitem__
             try:
                 perform(src, expected[strip_text], strip_text=strip_text)
             finally:
@@ -1095,10 +1088,8 @@ class TestTranslation(TestCase):
 
         # Build translation table
         messages = {"hi": "xi"}
-        for _, _, msgid, _ in i18n.extract(
-            BytesIO(src.encode("utf-8")), [], None, {"extract_python": True}
-        ):
-            messages[msgid] = "TRANSLATED(%s)" % msgid
+        for _, _, msgid, _ in i18n.extract(BytesIO(src.encode("utf-8")), [], None, {"extract_python": True}):
+            messages[msgid] = f"TRANSLATED({msgid})"
 
         # Provide a fake translation function
         default_gettext = i18n.gettext
@@ -1110,78 +1101,62 @@ class TestTranslation(TestCase):
 
     def test_extract_python_inside_invalid(self):
         src = """<xml><div>${_('hi' +)}</div></xml>"""
-        try:
-            list(
-                i18n.extract(
-                    BytesIO(src.encode("utf-8")), [], None, {"extract_python": True}
-                )
-            )
-        except XMLTemplateCompileError as e:
-            assert "_('hi' +)" in str(e)
-        else:
-            assert False, "Should have raised"
+        with pytest.raises(XMLTemplateCompileError, match=r"_\('hi' \+\)"):
+            list(i18n.extract(BytesIO(src.encode("utf-8")), [], None, {"extract_python": True}))
 
     def test_substituting_gettext_with_lambda(self):
         src = """<xml>hi</xml>"""
         expected = """<xml>spam</xml>"""
 
-        perform(src, expected, context=dict(gettext=lambda x: "spam"))
+        perform(src, expected, context={"gettext": lambda _: "spam"})
 
     def test_substituting_gettext_with_lambda_extending(self):
         loader = MockLoader(
             {
                 "parent.html": XMLTemplate("""<div>parent</div>"""),
-                "child.html": XMLTemplate(
-                    """<py:extends href="parent.html"><div>child</div></py:extends>"""
-                ),
+                "child.html": XMLTemplate("""<py:extends href="parent.html"><div>child</div></py:extends>"""),
             }
         )
         tpl = loader.import_("child.html")
-        rsp = tpl(dict(gettext=lambda _: "egg")).render()
+        rsp = tpl({"gettext": lambda _: "egg"}).render()
         assert rsp == """<div>egg</div><div>egg</div>""", rsp
 
     def test_substituting_gettext_with_lambda_extending_twice(self):
         loader = MockLoader(
             {
                 "parent.html": XMLTemplate("<div>parent</div>"),
-                "mid.html": XMLTemplate(
-                    '<py:extends href="parent.html"><div>${variable}</div></py:extends>'
-                ),
-                "child.html": XMLTemplate(
-                    '<py:extends href="mid.html"><div>child</div></py:extends>'
-                ),
+                "mid.html": XMLTemplate('<py:extends href="parent.html"><div>${variable}</div></py:extends>'),
+                "child.html": XMLTemplate('<py:extends href="mid.html"><div>child</div></py:extends>'),
             }
         )
         tpl = loader.import_("child.html")
-        rsp = tpl(dict(variable="spam", gettext=lambda _: "egg")).render()
+        rsp = tpl({"variable": "spam", "gettext": lambda _: "egg"}).render()
         # variables must not be translated
         assert rsp == """<div>egg</div><div>spam</div><div>egg</div>""", rsp
 
     def test_substituting_gettext_with_lambda_extending_file(self):
         loader = FileLoader(
             path=os.path.join(os.path.dirname(__file__), "data"),
-            base_globals=dict(gettext=lambda x: "egg"),
+            base_globals={"gettext": lambda _: "egg"},
         )
         tpl = loader.import_("file_child.html")
-        rsp = tpl(dict()).render()
+        rsp = tpl({}).render()
         assert rsp == """<div>egg</div><div>egg</div>""", rsp
 
     def test_without_substituting_gettext_with_lambda_extending_file(self):
         # this should use i18n.gettext
         loader = FileLoader(path=os.path.join(os.path.dirname(__file__), "data"))
         tpl = loader.import_("file_child.html")
-        rsp = tpl(dict()).render()
+        rsp = tpl({}).render()
         assert rsp == """<div>parent</div><div>child</div>""", rsp
 
 
 class TestDOMTransformations(TestCase):
     def test_empty_text_extraction(self):
-        doc = kajiki.xml_template._Parser(
-            "<string>", """<span>  text  </span>"""
-        ).parse()
+        doc = kajiki.xml_template._Parser("<string>", """<span>  text  </span>""").parse()
         doc = kajiki.xml_template._DomTransformer(doc, strip_text=False).transform()
         text_data = [n.data for n in doc.firstChild.childNodes]
-        self.assertEqual(["  ", "text", "  "], text_data)
+        assert ["  ", "text", "  "] == text_data
 
     def test_empty_text_extraction_lineno(self):
         doc = kajiki.xml_template._Parser(
@@ -1194,38 +1169,33 @@ class TestDOMTransformations(TestCase):
         ).parse()
         doc = kajiki.xml_template._DomTransformer(doc, strip_text=False).transform()
         linenos = [n.lineno for n in doc.firstChild.childNodes]
-        self.assertEqual(
-            [1, 3, 3], linenos
-        )  # Last node starts on same line as it starts with \n
+        assert [1, 3, 3] == linenos  # Last node starts on same line as it starts with \n
 
 
 class TestErrorReporting(TestCase):
     def test_syntax_error(self):
         for strip_text in (False, True):
-            try:
+            with pytest.raises(
+                KajikiSyntaxError,
+                match=r"-->         for i i range\(1, 2\):",
+            ):
                 perform(
                     '<div py:for="i i range(1, 2)">${i}</div>',
                     "",
                     strip_text=strip_text,
                 )
-            except KajikiSyntaxError as exc:
-                assert "-->         for i i range(1, 2):" in str(exc), exc
-            else:
-                assert False
 
+    @pytest.mark.skipif(sys.implementation.name == "pypy", reason="lnotab has issues with pypy")
     def test_code_error(self):
         for strip_text in (False, True):
-            try:
-                child = FileLoader(
-                    os.path.join(os.path.dirname(__file__), "data")
-                ).load("error.html", strip_text=strip_text)
+            child = FileLoader(os.path.join(os.path.dirname(__file__), "data")).load(
+                "error.html", strip_text=strip_text
+            )
+            with pytest.raises(ZeroDivisionError) as exc_info:
                 child().render()
-            except ZeroDivisionError:
-                exn_info = traceback.format_exception(*sys.exc_info())
-                last_line = exn_info[-2]
-                assert "${3/0}" in last_line
-            else:
-                assert False
+            formatted = traceback.format_exception(None, exc_info.value, exc_info.tb)
+            last_line = formatted[-2]
+            assert "${3/0}" in last_line
 
 
 class TestBracketsInExpression(TestCase):
@@ -1240,8 +1210,7 @@ class TestBracketsInExpression(TestCase):
 
     def test_complex(self):
         perform(
-            "<xml><div>${'ciao {  } {' + \"a {} b {{{{} w}}rar\"}${'sd{}'}"
-            " ${1+1}</div></xml>",
+            "<xml><div>${'ciao {  } {' + \"a {} b {{{{} w}}rar\"}${'sd{}'}" " ${1+1}</div></xml>",
             "<xml><div>ciao {  } {a {} b {{{{} w}}rarsd{} 2</div></xml>",
         )
 
@@ -1252,34 +1221,30 @@ class TestBracketsInExpression(TestCase):
         )
 
     def test_raise_unclosed_string(self):
-        try:
+        with pytest.raises(XMLTemplateCompileError) as e:
             XMLTemplate('<x>${"ciao}</x>')
-            assert False, "must raise"
-        except XMLTemplateCompileError as e:
-            # assert "can't compile" in str(e), e  # different between pypy and cpython
-            assert '"ciao' in str(e), e
+        # assert "can't compile" in str(e)  # different between pypy and cpython
+        assert '"ciao' in str(e)
 
     def test_raise_plus_with_an_operand(self):
-        try:
+        with pytest.raises(XMLTemplateCompileError) as e:
             XMLTemplate('<x>${"ciao" + }</x>')
-            assert False, "must raise"
-        except XMLTemplateCompileError as e:
-            assert "detected an invalid python expression" in str(e), e
-            assert '"ciao" +' in str(e), e
+        assert "detected an invalid python expression" in str(e)
+        assert '"ciao" +' in str(e)
 
     def test_unclosed_braced(self):
-        try:
+        with pytest.raises(
+            XMLTemplateCompileError,
+            match="Braced expression not terminated",
+        ):
             XMLTemplate('<x>${"ciao"</x>')
-            assert False, "must raise"
-        except XMLTemplateCompileError as e:
-            assert "Braced expression not terminated" in str(e), e
 
     def test_leading_opening_brace(self):
-        try:
+        with pytest.raises(
+            XMLTemplateCompileError,
+            match="Braced expression not terminated",
+        ):
             XMLTemplate('<x>${{"a", "b"}</x>')
-            assert False, "must raise"
-        except XMLTemplateCompileError as e:
-            assert "Braced expression not terminated" in str(e), e
 
 
 class TestMultipleChildrenInDOM(TestCase):
@@ -1291,34 +1256,33 @@ class TestMultipleChildrenInDOM(TestCase):
         assert res == "<x>2</x>"
 
     def test_multiple_nodes(self):
-        try:
+        with pytest.raises(XMLTemplateParseError, match="junk after document"):
             XMLTemplate("<!-- a --><x>${1+1}</x><y>${1+1}</y>")
-        except XMLTemplateParseError as e:
-            assert "junk after document element" in str(e), e
-        else:
-            assert False, "should have raised"
 
     def test_only_comment(self):
-        try:
+        with pytest.raises(XMLTemplateParseError, match="no element found"):
             XMLTemplate("<!-- a -->")
-        except XMLTemplateParseError as e:
-            assert "no element found" in str(e), e
-        else:
-            assert False, "should have raised"
 
 
 class TestSyntaxErrorCallingWithTrailingParenthesis(TestCase):
     def test_raise(self):
-        try:
+        with pytest.raises(XMLTemplateCompileError):
             XMLTemplate(
                 """<div py:strip="True"
 ><py:def function="echo(x)">$x</py:def
 >${echo('hello'))}</div>"""
             )
-            assert False, "should raise"
-        except XMLTemplateCompileError:
-            pass
 
 
-if __name__ == "__main__":
-    main()
+class TestExtendsWithImport(TestCase):
+    def test_extends_with_import(self):
+        loader = MockLoader(
+            {
+                "parent.html": XMLTemplate("<div>" '<py:import href="lib.html"/>' "${lib.foo()}" "</div>"),
+                "lib.html": XMLTemplate("<div>" '<py:def function="foo()"><b>foo</b></py:def>' "</div>"),
+                "child.html": XMLTemplate('<py:extends href="parent.html"/>'),
+            }
+        )
+        child = loader.import_("child.html")
+        r = child().render()
+        assert r == "<div><b>foo</b></div>"
